@@ -22,20 +22,21 @@ import java.util.*;
 import java.util.function.Consumer;
 import java.util.function.Predicate;
 import java.nio.ByteBuffer;
-import java.security.MessageDigest;
 
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.Iterators;
 
 import net.nicoulaj.compilecommand.annotations.DontInline;
-import org.apache.cassandra.config.CFMetaData;
-import org.apache.cassandra.config.ColumnDefinition;
+import org.apache.cassandra.exceptions.UnknownColumnException;
+import org.apache.cassandra.schema.ColumnMetadata;
+import org.apache.cassandra.schema.TableMetadata;
 import org.apache.cassandra.cql3.ColumnIdentifier;
 import org.apache.cassandra.db.marshal.SetType;
 import org.apache.cassandra.db.marshal.UTF8Type;
 import org.apache.cassandra.io.util.DataInputPlus;
 import org.apache.cassandra.io.util.DataOutputPlus;
 import org.apache.cassandra.utils.ByteBufferUtil;
+import org.apache.cassandra.utils.ObjectSizes;
 import org.apache.cassandra.utils.SearchIterator;
 import org.apache.cassandra.utils.btree.BTree;
 import org.apache.cassandra.utils.btree.BTreeSearchIterator;
@@ -46,28 +47,29 @@ import org.apache.cassandra.utils.btree.UpdateFunction;
  * An immutable and sorted list of (non-PK) columns for a given table.
  * <p>
  * Note that in practice, it will either store only static columns, or only regular ones. When
- * we need both type of columns, we use a {@link PartitionColumns} object.
+ * we need both type of columns, we use a {@link RegularAndStaticColumns} object.
  */
-public class Columns extends AbstractCollection<ColumnDefinition> implements Collection<ColumnDefinition>
+public class Columns extends AbstractCollection<ColumnMetadata> implements Collection<ColumnMetadata>
 {
     public static final Serializer serializer = new Serializer();
     public static final Columns NONE = new Columns(BTree.empty(), 0);
+    static final long EMPTY_SIZE = ObjectSizes.measure(NONE);
 
-    private static final ColumnDefinition FIRST_COMPLEX_STATIC =
-        new ColumnDefinition("",
-                             "",
-                             ColumnIdentifier.getInterned(ByteBufferUtil.EMPTY_BYTE_BUFFER, UTF8Type.instance),
-                             SetType.getInstance(UTF8Type.instance, true),
-                             ColumnDefinition.NO_POSITION,
-                             ColumnDefinition.Kind.STATIC);
+    public static final ColumnMetadata FIRST_COMPLEX_STATIC =
+        new ColumnMetadata("",
+                           "",
+                           ColumnIdentifier.getInterned(ByteBufferUtil.EMPTY_BYTE_BUFFER, UTF8Type.instance),
+                           SetType.getInstance(UTF8Type.instance, true),
+                           ColumnMetadata.NO_POSITION,
+                           ColumnMetadata.Kind.STATIC);
 
-    private static final ColumnDefinition FIRST_COMPLEX_REGULAR =
-        new ColumnDefinition("",
-                             "",
-                             ColumnIdentifier.getInterned(ByteBufferUtil.EMPTY_BYTE_BUFFER, UTF8Type.instance),
-                             SetType.getInstance(UTF8Type.instance, true),
-                             ColumnDefinition.NO_POSITION,
-                             ColumnDefinition.Kind.REGULAR);
+    public static final ColumnMetadata FIRST_COMPLEX_REGULAR =
+        new ColumnMetadata("",
+                           "",
+                           ColumnIdentifier.getInterned(ByteBufferUtil.EMPTY_BYTE_BUFFER, UTF8Type.instance),
+                           SetType.getInstance(UTF8Type.instance, true),
+                           ColumnMetadata.NO_POSITION,
+                           ColumnMetadata.Kind.REGULAR);
 
     private final Object[] columns;
     private final int complexIdx; // Index of the first complex column
@@ -91,7 +93,7 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
      *
      * @return the newly created {@code Columns} containing only {@code c}.
      */
-    public static Columns of(ColumnDefinition c)
+    public static Columns of(ColumnMetadata c)
     {
         return new Columns(BTree.singleton(c), c.isComplex() ? 0 : 1);
     }
@@ -102,9 +104,9 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
      * @param s the set from which to create the new {@code Columns}.
      * @return the newly created {@code Columns} containing the columns from {@code s}.
      */
-    public static Columns from(Collection<ColumnDefinition> s)
+    public static Columns from(Collection<ColumnMetadata> s)
     {
-        Object[] tree = BTree.<ColumnDefinition>builder(Comparator.naturalOrder()).addAll(s).build();
+        Object[] tree = BTree.<ColumnMetadata>builder(Comparator.naturalOrder()).addAll(s).build();
         return new Columns(tree, findFirstComplexIdx(tree));
     }
 
@@ -114,7 +116,7 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
             return 0;
 
         int size = BTree.size(tree);
-        ColumnDefinition last = BTree.findByIndex(tree, size - 1);
+        ColumnMetadata last = BTree.findByIndex(tree, size - 1);
         return last.isSimple()
              ? size
              : BTree.ceilIndex(tree, Comparator.naturalOrder(), last.isStatic() ? FIRST_COMPLEX_STATIC : FIRST_COMPLEX_REGULAR);
@@ -188,7 +190,7 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
      *
      * @return the {@code i}th simple column in this object.
      */
-    public ColumnDefinition getSimple(int i)
+    public ColumnMetadata getSimple(int i)
     {
         return BTree.findByIndex(columns, i);
     }
@@ -201,7 +203,7 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
      *
      * @return the {@code i}th complex column in this object.
      */
-    public ColumnDefinition getComplex(int i)
+    public ColumnMetadata getComplex(int i)
     {
         return BTree.findByIndex(columns, complexIdx + i);
     }
@@ -215,7 +217,7 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
      * @return the index for simple column {@code c} if it is contains in this
      * object
      */
-    public int simpleIdx(ColumnDefinition c)
+    public int simpleIdx(ColumnMetadata c)
     {
         return BTree.findIndex(columns, Comparator.naturalOrder(), c);
     }
@@ -229,7 +231,7 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
      * @return the index for complex column {@code c} if it is contains in this
      * object
      */
-    public int complexIdx(ColumnDefinition c)
+    public int complexIdx(ColumnMetadata c)
     {
         return BTree.findIndex(columns, Comparator.naturalOrder(), c) - complexIdx;
     }
@@ -241,7 +243,7 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
      *
      * @return whether {@code c} is contained by this object.
      */
-    public boolean contains(ColumnDefinition c)
+    public boolean contains(ColumnMetadata c)
     {
         return BTree.findIndex(columns, Comparator.naturalOrder(), c) >= 0;
     }
@@ -263,8 +265,8 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
         if (this == NONE)
             return other;
 
-        Object[] tree = BTree.<ColumnDefinition>merge(this.columns, other.columns, Comparator.naturalOrder(),
-                                                      UpdateFunction.noOp());
+        Object[] tree = BTree.<ColumnMetadata>merge(this.columns, other.columns, Comparator.naturalOrder(),
+                                                    UpdateFunction.noOp());
         if (tree == this.columns)
             return this;
         if (tree == other.columns)
@@ -287,9 +289,9 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
         if (other.size() > this.size())
             return false;
 
-        BTreeSearchIterator<ColumnDefinition, ColumnDefinition> iter = BTree.slice(columns, Comparator.naturalOrder(), BTree.Dir.ASC);
+        BTreeSearchIterator<ColumnMetadata, ColumnMetadata> iter = BTree.slice(columns, Comparator.naturalOrder(), BTree.Dir.ASC);
         for (Object def : other)
-            if (iter.next((ColumnDefinition) def) == null)
+            if (iter.next((ColumnMetadata) def) == null)
                 return false;
         return true;
     }
@@ -299,7 +301,7 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
      *
      * @return an iterator over the simple columns of this object.
      */
-    public Iterator<ColumnDefinition> simpleColumns()
+    public Iterator<ColumnMetadata> simpleColumns()
     {
         return BTree.iterator(columns, 0, complexIdx - 1, BTree.Dir.ASC);
     }
@@ -309,7 +311,7 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
      *
      * @return an iterator over the complex columns of this object.
      */
-    public Iterator<ColumnDefinition> complexColumns()
+    public Iterator<ColumnMetadata> complexColumns()
     {
         return BTree.iterator(columns, complexIdx, BTree.size(columns) - 1, BTree.Dir.ASC);
     }
@@ -319,9 +321,9 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
      *
      * @return an iterator over all the columns of this object.
      */
-    public BTreeSearchIterator<ColumnDefinition, ColumnDefinition> iterator()
+    public BTreeSearchIterator<ColumnMetadata, ColumnMetadata> iterator()
     {
-        return BTree.<ColumnDefinition, ColumnDefinition>slice(columns, Comparator.naturalOrder(), BTree.Dir.ASC);
+        return BTree.<ColumnMetadata, ColumnMetadata>slice(columns, Comparator.naturalOrder(), BTree.Dir.ASC);
     }
 
     /**
@@ -331,11 +333,11 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
      *
      * @return an iterator returning columns in alphabetical order.
      */
-    public Iterator<ColumnDefinition> selectOrderIterator()
+    public Iterator<ColumnMetadata> selectOrderIterator()
     {
         // In wildcard selection, we want to return all columns in alphabetical order,
         // irregarding of whether they are complex or not
-        return Iterators.<ColumnDefinition>
+        return Iterators.<ColumnMetadata>
                          mergeSorted(ImmutableList.of(simpleColumns(), complexColumns()),
                                      (s, c) ->
                                      {
@@ -352,12 +354,12 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
      * @return newly allocated columns containing all the columns of {@code this} expect
      * for {@code column}.
      */
-    public Columns without(ColumnDefinition column)
+    public Columns without(ColumnMetadata column)
     {
         if (!contains(column))
             return this;
 
-        Object[] newColumns = BTreeRemoval.<ColumnDefinition>remove(columns, Comparator.naturalOrder(), column);
+        Object[] newColumns = BTreeRemoval.<ColumnMetadata>remove(columns, Comparator.naturalOrder(), column);
         return new Columns(newColumns);
     }
 
@@ -367,33 +369,25 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
      *
      * @return a predicate to test the inclusion of sorted columns in this object.
      */
-    public Predicate<ColumnDefinition> inOrderInclusionTester()
+    public Predicate<ColumnMetadata> inOrderInclusionTester()
     {
-        SearchIterator<ColumnDefinition, ColumnDefinition> iter = BTree.slice(columns, Comparator.naturalOrder(), BTree.Dir.ASC);
+        SearchIterator<ColumnMetadata, ColumnMetadata> iter = BTree.slice(columns, Comparator.naturalOrder(), BTree.Dir.ASC);
         return column -> iter.next(column) != null;
     }
 
-    public void digest(MessageDigest digest)
+    public void digest(Digest digest)
     {
-        for (ColumnDefinition c : this)
-            digest.update(c.name.bytes.duplicate());
-    }
-
-    public void digest(MessageDigest digest, Set<ByteBuffer> columnsToExclude)
-    {
-        for (ColumnDefinition c : this)
-            if (!columnsToExclude.contains(c.name.bytes))
-                digest.update(c.name.bytes.duplicate());
+        for (ColumnMetadata c : this)
+            digest.update(c.name.bytes);
     }
 
     /**
      * Apply a function to each column definition in forwards or reversed order.
      * @param function
-     * @param reversed
      */
-    public void apply(Consumer<ColumnDefinition> function, boolean reversed)
+    public void apply(Consumer<ColumnMetadata> function)
     {
-        BTree.apply(columns, function, reversed);
+        BTree.apply(columns, function);
     }
 
     @Override
@@ -414,12 +408,20 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
         return Objects.hash(complexIdx, BTree.hashCode(columns));
     }
 
+    public long unsharedHeapSize()
+    {
+        if(this == NONE)
+            return 0;
+
+        return EMPTY_SIZE;
+    }
+
     @Override
     public String toString()
     {
         StringBuilder sb = new StringBuilder("[");
         boolean first = true;
-        for (ColumnDefinition def : this)
+        for (ColumnMetadata def : this)
         {
             if (first) first = false; else sb.append(" ");
             sb.append(def.name);
@@ -432,34 +434,33 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
         public void serialize(Columns columns, DataOutputPlus out) throws IOException
         {
             out.writeUnsignedVInt(columns.size());
-            for (ColumnDefinition column : columns)
+            for (ColumnMetadata column : columns)
                 ByteBufferUtil.writeWithVIntLength(column.name.bytes, out);
         }
 
         public long serializedSize(Columns columns)
         {
             long size = TypeSizes.sizeofUnsignedVInt(columns.size());
-            for (ColumnDefinition column : columns)
+            for (ColumnMetadata column : columns)
                 size += ByteBufferUtil.serializedSizeWithVIntLength(column.name.bytes);
             return size;
         }
 
-        public Columns deserialize(DataInputPlus in, CFMetaData metadata) throws IOException
+        public Columns deserialize(DataInputPlus in, TableMetadata metadata) throws IOException
         {
             int length = (int)in.readUnsignedVInt();
-            BTree.Builder<ColumnDefinition> builder = BTree.builder(Comparator.naturalOrder());
+            BTree.Builder<ColumnMetadata> builder = BTree.builder(Comparator.naturalOrder());
             builder.auto(false);
             for (int i = 0; i < length; i++)
             {
                 ByteBuffer name = ByteBufferUtil.readWithVIntLength(in);
-                ColumnDefinition column = metadata.getColumnDefinition(name);
-
+                ColumnMetadata column = metadata.getColumn(name);
                 if (column == null)
                 {
                     // If we don't find the definition, it could be we have data for a dropped column, and we shouldn't
-                    // fail deserialization because of that. So we grab a "fake" ColumnDefinition that ensure proper
+                    // fail deserialization because of that. So we grab a "fake" ColumnMetadata that ensure proper
                     // deserialization. The column will be ignore later on anyway.
-                    column = metadata.getDroppedColumnDefinition(name);
+                    column = metadata.getDroppedColumn(name);
 
                     if (column == null)
                         throw new RuntimeException("Unknown column " + UTF8Type.instance.getString(name) + " during deserialization");
@@ -473,7 +474,7 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
          * If both ends have a pre-shared superset of the columns we are serializing, we can send them much
          * more efficiently. Both ends must provide the identically same set of columns.
          */
-        public void serializeSubset(Collection<ColumnDefinition> columns, Columns superset, DataOutputPlus out) throws IOException
+        public void serializeSubset(Collection<ColumnMetadata> columns, Columns superset, DataOutputPlus out) throws IOException
         {
             /**
              * We weight this towards small sets, and sets where the majority of items are present, since
@@ -503,7 +504,7 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
             }
         }
 
-        public long serializedSubsetSize(Collection<ColumnDefinition> columns, Columns superset)
+        public long serializedSubsetSize(Collection<ColumnMetadata> columns, Columns superset)
         {
             int columnCount = columns.size();
             int supersetCount = superset.size();
@@ -534,9 +535,9 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
             }
             else
             {
-                BTree.Builder<ColumnDefinition> builder = BTree.builder(Comparator.naturalOrder());
+                BTree.Builder<ColumnMetadata> builder = BTree.builder(Comparator.naturalOrder());
                 int firstComplexIdx = 0;
-                for (ColumnDefinition column : superset)
+                for (ColumnMetadata column : superset)
                 {
                     if ((encoded & 1) == 0)
                     {
@@ -554,13 +555,13 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
 
         // encodes a 1 bit for every *missing* column, on the assumption presence is more common,
         // and because this is consistent with encoding 0 to represent all present
-        private static long encodeBitmap(Collection<ColumnDefinition> columns, Columns superset, int supersetCount)
+        private static long encodeBitmap(Collection<ColumnMetadata> columns, Columns superset, int supersetCount)
         {
             long bitmap = 0L;
-            BTreeSearchIterator<ColumnDefinition, ColumnDefinition> iter = superset.iterator();
+            BTreeSearchIterator<ColumnMetadata, ColumnMetadata> iter = superset.iterator();
             // the index we would encounter next if all columns are present
             int expectIndex = 0;
-            for (ColumnDefinition column : columns)
+            for (ColumnMetadata column : columns)
             {
                 if (iter.next(column) == null)
                     throw new IllegalStateException(columns + " is not a subset of " + superset);
@@ -579,15 +580,15 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
         }
 
         @DontInline
-        private void serializeLargeSubset(Collection<ColumnDefinition> columns, int columnCount, Columns superset, int supersetCount, DataOutputPlus out) throws IOException
+        private void serializeLargeSubset(Collection<ColumnMetadata> columns, int columnCount, Columns superset, int supersetCount, DataOutputPlus out) throws IOException
         {
             // write flag indicating we're in lengthy mode
             out.writeUnsignedVInt(supersetCount - columnCount);
-            BTreeSearchIterator<ColumnDefinition, ColumnDefinition> iter = superset.iterator();
+            BTreeSearchIterator<ColumnMetadata, ColumnMetadata> iter = superset.iterator();
             if (columnCount < supersetCount / 2)
             {
                 // write present columns
-                for (ColumnDefinition column : columns)
+                for (ColumnMetadata column : columns)
                 {
                     if (iter.next(column) == null)
                         throw new IllegalStateException();
@@ -598,7 +599,7 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
             {
                 // write missing columns
                 int prev = -1;
-                for (ColumnDefinition column : columns)
+                for (ColumnMetadata column : columns)
                 {
                     if (iter.next(column) == null)
                         throw new IllegalStateException();
@@ -617,7 +618,7 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
             int supersetCount = superset.size();
             int columnCount = supersetCount - delta;
 
-            BTree.Builder<ColumnDefinition> builder = BTree.builder(Comparator.naturalOrder());
+            BTree.Builder<ColumnMetadata> builder = BTree.builder(Comparator.naturalOrder());
             if (columnCount < supersetCount / 2)
             {
                 for (int i = 0 ; i < columnCount ; i++)
@@ -628,7 +629,7 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
             }
             else
             {
-                Iterator<ColumnDefinition> iter = superset.iterator();
+                Iterator<ColumnMetadata> iter = superset.iterator();
                 int idx = 0;
                 int skipped = 0;
                 while (true)
@@ -636,7 +637,7 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
                     int nextMissingIndex = skipped < delta ? (int)in.readUnsignedVInt() : supersetCount;
                     while (idx < nextMissingIndex)
                     {
-                        ColumnDefinition def = iter.next();
+                        ColumnMetadata def = iter.next();
                         builder.add(def);
                         idx++;
                     }
@@ -651,15 +652,15 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
         }
 
         @DontInline
-        private int serializeLargeSubsetSize(Collection<ColumnDefinition> columns, int columnCount, Columns superset, int supersetCount)
+        private int serializeLargeSubsetSize(Collection<ColumnMetadata> columns, int columnCount, Columns superset, int supersetCount)
         {
             // write flag indicating we're in lengthy mode
             int size = TypeSizes.sizeofUnsignedVInt(supersetCount - columnCount);
-            BTreeSearchIterator<ColumnDefinition, ColumnDefinition> iter = superset.iterator();
+            BTreeSearchIterator<ColumnMetadata, ColumnMetadata> iter = superset.iterator();
             if (columnCount < supersetCount / 2)
             {
                 // write present columns
-                for (ColumnDefinition column : columns)
+                for (ColumnMetadata column : columns)
                 {
                     if (iter.next(column) == null)
                         throw new IllegalStateException();
@@ -670,7 +671,7 @@ public class Columns extends AbstractCollection<ColumnDefinition> implements Col
             {
                 // write missing columns
                 int prev = -1;
-                for (ColumnDefinition column : columns)
+                for (ColumnMetadata column : columns)
                 {
                     if (iter.next(column) == null)
                         throw new IllegalStateException();
