@@ -21,8 +21,6 @@ import java.io.InputStream;
 import java.net.URL;
 import java.util.Properties;
 
-import com.google.common.annotations.VisibleForTesting;
-
 import org.apache.cassandra.io.util.FileUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -32,7 +30,7 @@ public class SnitchProperties
     private static final Logger logger = LoggerFactory.getLogger(SnitchProperties.class);
     public static final String RACKDC_PROPERTY_FILENAME = "cassandra-rackdc.properties";
 
-    private Properties properties;
+    private final Properties properties;
 
     public SnitchProperties()
     {
@@ -61,12 +59,6 @@ public class SnitchProperties
         }
     }
 
-    @VisibleForTesting
-    public SnitchProperties(Properties properties)
-    {
-        this.properties = properties;
-    }
-
     /**
      * Get a snitch property value or return defaultValue if not defined.
      */
