@@ -18,11 +18,14 @@
  */
 package org.apache.cassandra.tools;
 
-import org.apache.cassandra.config.Schema;
+import org.apache.cassandra.schema.Schema;
 import org.apache.cassandra.db.ColumnFamilyStore;
 import org.apache.cassandra.db.Directories;
 import org.apache.cassandra.db.Keyspace;
 import org.apache.cassandra.db.compaction.*;
+import org.apache.cassandra.dht.Murmur3Partitioner;
+import org.apache.cassandra.dht.Range;
+import org.apache.cassandra.dht.Token;
 import org.apache.cassandra.io.sstable.Component;
 import org.apache.cassandra.io.sstable.CorruptSSTableException;
 import org.apache.cassandra.io.sstable.Descriptor;
@@ -33,6 +36,8 @@ import org.apache.commons.cli.*;
 
 import java.util.*;
 import java.util.concurrent.TimeUnit;
+import java.util.stream.Collectors;
+import java.util.stream.Stream;
 
 import static org.apache.cassandra.tools.BulkLoader.CmdLineOptions;
 
@@ -43,11 +48,16 @@ public class StandaloneVerifier
     private static final String EXTENDED_OPTION = "extended";
     private static final String DEBUG_OPTION  = "debug";
     private static final String HELP_OPTION  = "help";
+    private static final String CHECK_VERSION = "check_version";
+    private static final String MUTATE_REPAIR_STATUS = "mutate_repair_status";
+    private static final String QUICK = "quick";
+    private static final String TOKEN_RANGE = "token_range";
 
     public static void main(String args[])
     {
         Options options = Options.parseArgs(args);
         Util.initDatabaseDescriptor();
+        System.out.println("sstableverify using the following options: " + options);
 
         try
         {
@@ -56,7 +66,7 @@ public class StandaloneVerifier
 
             boolean hasFailed = false;
 
-            if (Schema.instance.getCFMetaData(options.keyspaceName, options.cfName) == null)
+            if (Schema.instance.getTableMetadataRef(options.keyspaceName, options.cfName) == null)
                 throw new IllegalArgumentException(String.format("Unknown keyspace/table %s.%s",
                                                                  options.keyspaceName,
                                                                  options.cfName));
@@ -68,8 +78,6 @@ public class StandaloneVerifier
             OutputHandler handler = new OutputHandler.SystemOutput(options.verbose, options.debug);
             Directories.SSTableLister lister = cfs.getDirectories().sstableLister(Directories.OnTxnErr.THROW).skipTemporary(true);
 
-            boolean extended = options.extended;
-
             List<SSTableReader> sstables = new ArrayList<>();
 
             // Verify sstables
@@ -92,12 +100,19 @@ public class StandaloneVerifier
                         e.printStackTrace(System.err);
                 }
             }
-
+            Verifier.Options verifyOptions = Verifier.options().invokeDiskFailurePolicy(false)
+                                                               .extendedVerification(options.extended)
+                                                               .checkVersion(options.checkVersion)
+                                                               .mutateRepairStatus(options.mutateRepairStatus)
+                                                               .checkOwnsTokens(!options.tokens.isEmpty())
+                                                               .tokenLookup(ignore -> options.tokens)
+                                                               .build();
+            handler.output("Running verifier with the following options: " + verifyOptions);
             for (SSTableReader sstable : sstables)
             {
-                try (Verifier verifier = new Verifier(cfs, sstable, handler, true))
+                try (Verifier verifier = new Verifier(cfs, sstable, handler, true, verifyOptions))
                 {
-                    verifier.verify(extended);
+                    verifier.verify();
                 }
                 catch (Exception e)
                 {
@@ -127,6 +142,10 @@ public class StandaloneVerifier
         public boolean debug;
         public boolean verbose;
         public boolean extended;
+        public boolean checkVersion;
+        public boolean mutateRepairStatus;
+        public boolean quick;
+        public Collection<Range<Token>> tokens;
 
         private Options(String keyspaceName, String cfName)
         {
@@ -165,6 +184,20 @@ public class StandaloneVerifier
                 opts.debug = cmd.hasOption(DEBUG_OPTION);
                 opts.verbose = cmd.hasOption(VERBOSE_OPTION);
                 opts.extended = cmd.hasOption(EXTENDED_OPTION);
+                opts.checkVersion = cmd.hasOption(CHECK_VERSION);
+                opts.mutateRepairStatus = cmd.hasOption(MUTATE_REPAIR_STATUS);
+                opts.quick = cmd.hasOption(QUICK);
+
+                if (cmd.hasOption(TOKEN_RANGE))
+                {
+                    opts.tokens = Stream.of(cmd.getOptionValues(TOKEN_RANGE))
+                                        .map(StandaloneVerifier::parseTokenRange)
+                                        .collect(Collectors.toSet());
+                }
+                else
+                {
+                    opts.tokens = Collections.emptyList();
+                }
 
                 return opts;
             }
@@ -175,6 +208,21 @@ public class StandaloneVerifier
             }
         }
 
+        public String toString()
+        {
+            return "Options{" +
+                   "keyspaceName='" + keyspaceName + '\'' +
+                   ", cfName='" + cfName + '\'' +
+                   ", debug=" + debug +
+                   ", verbose=" + verbose +
+                   ", extended=" + extended +
+                   ", checkVersion=" + checkVersion +
+                   ", mutateRepairStatus=" + mutateRepairStatus +
+                   ", quick=" + quick +
+                   ", tokens=" + tokens +
+                   '}';
+        }
+
         private static void errorMsg(String msg, CmdLineOptions options)
         {
             System.err.println(msg);
@@ -189,6 +237,10 @@ public class StandaloneVerifier
             options.addOption("e",  EXTENDED_OPTION,       "extended verification");
             options.addOption("v",  VERBOSE_OPTION,        "verbose output");
             options.addOption("h",  HELP_OPTION,           "display this help message");
+            options.addOption("c",  CHECK_VERSION,         "make sure sstables are the latest version");
+            options.addOption("r",  MUTATE_REPAIR_STATUS,  "don't mutate repair status");
+            options.addOption("q",  QUICK,                 "do a quick check, don't read all data");
+            options.addOptionList("t", TOKEN_RANGE, "range", "long token range of the format left,right. This may be provided multiple times to define multiple different ranges");
             return options;
         }
 
@@ -203,4 +255,14 @@ public class StandaloneVerifier
             new HelpFormatter().printHelp(usage, header.toString(), options, "");
         }
     }
+
+    private static Range<Token> parseTokenRange(String line)
+    {
+        String[] split = line.split(",");
+        if (split.length != 2)
+            throw new IllegalArgumentException("Unable to parse token range from " + line + "; format is left,right but saw " + split.length + " parts");
+        long left = Long.parseLong(split[0]);
+        long right = Long.parseLong(split[1]);
+        return new Range<>(new Murmur3Partitioner.LongToken(left), new Murmur3Partitioner.LongToken(right));
+    }
 }
