@@ -36,6 +36,7 @@ import org.apache.cassandra.distributed.api.Feature;
 import org.apache.cassandra.distributed.api.IInstanceConfig;
 import org.apache.cassandra.distributed.shared.NetworkTopology;
 import org.apache.cassandra.distributed.shared.Shared;
+import org.apache.cassandra.distributed.upgrade.UpgradeTestBase;
 import org.apache.cassandra.locator.InetAddressAndPort;
 import org.apache.cassandra.locator.SimpleSeedProvider;
 
@@ -46,8 +47,6 @@ public class InstanceConfig implements IInstanceConfig
     private static final Logger logger = LoggerFactory.getLogger(InstanceConfig.class);
 
     public final int num;
-    private final int jmxPort;
-
     public int num() { return num; }
 
     private final NetworkTopology networkTopology;
@@ -69,13 +68,15 @@ public class InstanceConfig implements IInstanceConfig
                            String broadcast_rpc_address,
                            String rpc_address,
                            String seedIp,
+                           int seedPort,
                            String saved_caches_directory,
                            String[] data_file_directories,
                            String commitlog_directory,
                            String hints_directory,
                            String cdc_raw_directory,
                            String initial_token,
-                           int jmx_port)
+                           int storage_port,
+                           int native_transport_port)
     {
         this.num = num;
         this.networkTopology = networkTopology;
@@ -101,10 +102,13 @@ public class InstanceConfig implements IInstanceConfig
                 .set("concurrent_compactors", 1)
                 .set("memtable_heap_space_in_mb", 10)
                 .set("commitlog_sync", "batch")
-                .set("storage_port", 7012)
+                .set("storage_port", storage_port)
+                .set("native_transport_port", native_transport_port)
                 .set("endpoint_snitch", DistributedTestSnitch.class.getName())
                 .set("seed_provider", new ParameterizedClass(SimpleSeedProvider.class.getName(),
-                        Collections.singletonMap("seeds", seedIp)))
+                        Collections.singletonMap("seeds", seedIp + ":" + seedPort)))
+                // required settings for dtest functionality
+                .set("diagnostic_events_enabled", true)
                 .set("auto_bootstrap", false)
                 // capacities that are based on `totalMemory` that should be fixed size
                 .set("index_summary_capacity_in_mb", 50l)
@@ -113,7 +117,6 @@ public class InstanceConfig implements IInstanceConfig
                 // legacy parameters
                 .forceSet("commitlog_sync_batch_window_in_ms", 1.0);
         this.featureFlags = EnumSet.noneOf(Feature.class);
-        this.jmxPort = jmx_port;
     }
 
     private InstanceConfig(InstanceConfig copy)
@@ -125,7 +128,6 @@ public class InstanceConfig implements IInstanceConfig
         this.hostId = copy.hostId;
         this.featureFlags = copy.featureFlags;
         this.broadcastAddressAndPort = copy.broadcastAddressAndPort;
-        this.jmxPort = copy.jmxPort;
     }
 
 
@@ -135,6 +137,11 @@ public class InstanceConfig implements IInstanceConfig
         return DistributedTestSnitch.fromCassandraInetAddressAndPort(getBroadcastAddressAndPort());
     }
 
+    public void unsetBroadcastAddressAndPort()
+    {
+        broadcastAddressAndPort = null;
+    }
+
     protected InetAddressAndPort getBroadcastAddressAndPort()
     {
         if (broadcastAddressAndPort == null)
@@ -166,12 +173,6 @@ public class InstanceConfig implements IInstanceConfig
         return networkTopology().localDC(broadcastAddress());
     }
 
-    @Override
-    public int jmxPort()
-    {
-        return this.jmxPort;
-    }
-
     public InstanceConfig with(Feature featureFlag)
     {
         featureFlags.add(featureFlag);
@@ -194,12 +195,11 @@ public class InstanceConfig implements IInstanceConfig
     {
         if (value == null)
             value = NULL;
-
         getParams(fieldName).put(fieldName, value);
         return this;
     }
 
-    public InstanceConfig forceSet(String fieldName, Object value)
+    private InstanceConfig forceSet(String fieldName, Object value)
     {
         if (value == null)
             value = NULL;
@@ -248,22 +248,29 @@ public class InstanceConfig implements IInstanceConfig
         return params;
     }
 
-    public static InstanceConfig generate(int nodeNum, String ipAddress, NetworkTopology networkTopology, File root, String token, String seedIp, int datadirCount)
+    public static InstanceConfig generate(int nodeNum,
+                                          INodeProvisionStrategy provisionStrategy,
+                                          NetworkTopology networkTopology,
+                                          File root,
+                                          String token,
+                                          int datadirCount)
     {
         return new InstanceConfig(nodeNum,
                                   networkTopology,
-                                  ipAddress,
-                                  ipAddress,
-                                  ipAddress,
-                                  ipAddress,
-                                  seedIp,
+                                  provisionStrategy.ipAddress(nodeNum),
+                                  provisionStrategy.ipAddress(nodeNum),
+                                  provisionStrategy.ipAddress(nodeNum),
+                                  provisionStrategy.ipAddress(nodeNum),
+                                  provisionStrategy.seedIp(),
+                                  provisionStrategy.seedPort(),
                                   String.format("%s/node%d/saved_caches", root, nodeNum),
                                   datadirs(datadirCount, root, nodeNum),
                                   String.format("%s/node%d/commitlog", root, nodeNum),
                                   String.format("%s/node%d/hints", root, nodeNum),
                                   String.format("%s/node%d/cdc", root, nodeNum),
                                   token,
-                                  7199);
+                                  provisionStrategy.storagePort(nodeNum),
+                                  provisionStrategy.nativeTransportPort(nodeNum));
     }
 
     private static String[] datadirs(int datadirCount, File root, int nodeNum)
@@ -277,7 +284,11 @@ public class InstanceConfig implements IInstanceConfig
 
     public InstanceConfig forVersion(Semver version)
     {
-        return new InstanceConfig(this)
+        // Versions before 4.0 need to set 'seed_provider' without specifying the port
+        if (UpgradeTestBase.v40.compareTo(version) < 0)
+            return this;
+        else
+            return new InstanceConfig(this)
                             .set("seed_provider", new ParameterizedClass(SimpleSeedProvider.class.getName(),
                                                                          Collections.singletonMap("seeds", "127.0.0.1")));
     }
