@@ -18,11 +18,9 @@
 
 package org.apache.cassandra.db.compaction;
 
-import java.io.IOException;
+import java.util.concurrent.TimeUnit;
 import java.util.Collection;
 import java.util.Collections;
-import java.util.HashSet;
-import java.util.Set;
 import java.util.function.Consumer;
 import java.util.stream.Collectors;
 
@@ -35,10 +33,9 @@ import org.apache.cassandra.db.Keyspace;
 import org.apache.cassandra.dht.Range;
 import org.apache.cassandra.dht.Token;
 import org.apache.cassandra.io.sstable.Descriptor;
-import org.apache.cassandra.io.sstable.format.SSTableReader;
-import org.apache.cassandra.metrics.CompactionMetrics;
 import org.apache.cassandra.utils.FBUtilities;
 import org.jboss.byteman.contrib.bmunit.BMRule;
+import org.jboss.byteman.contrib.bmunit.BMRules;
 import org.jboss.byteman.contrib.bmunit.BMUnitRunner;
 
 import static org.junit.Assert.assertEquals;
@@ -48,6 +45,74 @@ import static org.junit.Assert.fail;
 @RunWith(BMUnitRunner.class)
 public class CompactionsBytemanTest extends CQLTester
 {
+    /*
+    Return false for the first time hasAvailableDiskSpace is called. i.e first SSTable is too big
+    Create 5 SSTables. After compaction, there should be 2 left - 1 as the 9 SStables which were merged,
+    and the other the SSTable that was 'too large' and failed the hasAvailableDiskSpace check
+     */
+    @Test
+    @BMRules(rules = { @BMRule(name = "One SSTable too big for remaining disk space test",
+    targetClass = "Directories",
+    targetMethod = "hasAvailableDiskSpace",
+    condition = "not flagged(\"done\")",
+    action = "flag(\"done\"); return false;") } )
+    public void testSSTableNotEnoughDiskSpaceForCompactionGetsDropped() throws Throwable
+    {
+        createLowGCGraceTable();
+        final ColumnFamilyStore cfs = getCurrentColumnFamilyStore();
+        for (int i = 0; i < 5; i++)
+        {
+            createPossiblyExpiredSSTable(cfs, false);
+        }
+        assertEquals(5, getCurrentColumnFamilyStore().getLiveSSTables().size());
+        cfs.forceMajorCompaction(false);
+        assertEquals(2, getCurrentColumnFamilyStore().getLiveSSTables().size());
+        dropTable("DROP TABLE %s");
+    }
+
+    /*
+    Always return false for hasAvailableDiskSpace. i.e node has no more space
+    Create 2 expired SSTables and 1 long lived one. After compaction, there should only be 1 left,
+    as the 2 expired ones would have been compacted away.
+     */
+    @Test
+    @BMRules(rules = { @BMRule(name = "No disk space with expired SSTables test",
+    targetClass = "Directories",
+    targetMethod = "hasAvailableDiskSpace",
+    action = "return false;") } )
+    public void testExpiredSSTablesStillGetDroppedWithNoDiskSpace() throws Throwable
+    {
+        createLowGCGraceTable();
+        final ColumnFamilyStore cfs = getCurrentColumnFamilyStore();
+        createPossiblyExpiredSSTable(cfs, true);
+        createPossiblyExpiredSSTable(cfs, true);
+        createPossiblyExpiredSSTable(cfs, false);
+        assertEquals(3, cfs.getLiveSSTables().size());
+        Thread.sleep(TimeUnit.SECONDS.toMillis((long)1.5)); // give some time to expire.
+        cfs.forceMajorCompaction(false);
+        assertEquals(1, cfs.getLiveSSTables().size());
+        dropTable("DROP TABLE %s");
+    }
+
+    /*
+    Always return false for hasAvailableDiskSpace. i.e node has no more space
+    Create 2 SSTables. Compaction will not succeed and will throw Runtime Exception
+     */
+    @Test(expected = RuntimeException.class)
+    @BMRules(rules = { @BMRule(name = "No disk space with expired SSTables test",
+    targetClass = "Directories",
+    targetMethod = "hasAvailableDiskSpace",
+    action = "return false;") } )
+    public void testRuntimeExceptionWhenNoDiskSpaceForCompaction() throws Throwable
+    {
+        createLowGCGraceTable();
+        final ColumnFamilyStore cfs = getCurrentColumnFamilyStore();
+        createPossiblyExpiredSSTable(cfs, false);
+        createPossiblyExpiredSSTable(cfs, false);
+        cfs.forceMajorCompaction(false);
+        dropTable("DROP TABLE %s");
+    }
+
     @Test
     @BMRule(name = "Delay background compaction task future check",
             targetClass = "CompactionManager",
@@ -57,8 +122,8 @@ public class CompactionsBytemanTest extends CQLTester
             action = "Thread.sleep(1000)")
     public void testCompactingCFCounting() throws Throwable
     {
-        String table = createTable("CREATE TABLE %s (k INT, c INT, v INT, PRIMARY KEY (k, c))");
-        ColumnFamilyStore cfs = Keyspace.open(CQLTester.KEYSPACE).getColumnFamilyStore(table);
+        createTable("CREATE TABLE %s (k INT, c INT, v INT, PRIMARY KEY (k, c))");
+        ColumnFamilyStore cfs = getCurrentColumnFamilyStore();
         cfs.enableAutoCompaction();
 
         execute("INSERT INTO %s (k, c, v) VALUES (?, ?, ?)", 0, 1, 1);
@@ -69,6 +134,24 @@ public class CompactionsBytemanTest extends CQLTester
         assertEquals(0, CompactionManager.instance.compactingCF.count(cfs));
     }
 
+    private void createPossiblyExpiredSSTable(final ColumnFamilyStore cfs, final boolean expired) throws Throwable
+    {
+        if (expired)
+        {
+            execute("INSERT INTO %s (id, val) values (1, 'expired') USING TTL 1");
+            Thread.sleep(TimeUnit.SECONDS.toMillis((long)1.5));
+        }
+        else
+        {
+            execute("INSERT INTO %s (id, val) values (2, 'immortal')");
+        }
+        cfs.forceBlockingFlush();
+    }
+
+    private void createLowGCGraceTable(){
+        createTable("CREATE TABLE %s (id int PRIMARY KEY, val text) with compaction = {'class':'SizeTieredCompactionStrategy', 'enabled': 'false'} AND gc_grace_seconds=0");
+    }
+
     @Test
     @BMRule(name = "Stop all compactions",
     targetClass = "CompactionTask",
@@ -111,7 +194,7 @@ public class CompactionsBytemanTest extends CQLTester
             }
             cfs.forceBlockingFlush();
         }
-        setRepaired(cfs, cfs.getLiveSSTables());
+        cfs.getCompactionStrategyManager().mutateRepaired(cfs.getLiveSSTables(), System.currentTimeMillis(), null, false);
         for (int i = 0; i < 5; i++)
         {
             for (int j = 0; j < 10; j++)
@@ -122,7 +205,7 @@ public class CompactionsBytemanTest extends CQLTester
         }
 
         assertTrue(cfs.getTracker().getCompacting().isEmpty());
-        assertTrue(CompactionMetrics.getCompactions().stream().noneMatch(h -> h.getCompactionInfo().getCFMetaData().equals(cfs.metadata)));
+        assertTrue(CompactionManager.instance.active.getCompactions().stream().noneMatch(h -> h.getCompactionInfo().getTableMetadata().equals(cfs.metadata)));
 
         try
         {
@@ -137,19 +220,7 @@ public class CompactionsBytemanTest extends CQLTester
         }
 
         assertTrue(cfs.getTracker().getCompacting().isEmpty());
-        assertTrue(CompactionMetrics.getCompactions().stream().noneMatch(h -> h.getCompactionInfo().getCFMetaData().equals(cfs.metadata)));
+        assertTrue(CompactionManager.instance.active.getCompactions().stream().noneMatch(h -> h.getCompactionInfo().getTableMetadata().equals(cfs.metadata)));
 
     }
-
-    private void setRepaired(ColumnFamilyStore cfs, Iterable<SSTableReader> sstables) throws IOException
-    {
-        Set<SSTableReader> changed = new HashSet<>();
-        for (SSTableReader sstable: sstables)
-        {
-            sstable.descriptor.getMetadataSerializer().mutateRepairedAt(sstable.descriptor, System.currentTimeMillis());
-            sstable.reloadSSTableMetadata();
-            changed.add(sstable);
-        }
-        cfs.getTracker().notifySSTableRepairedStatusChanged(changed);
-    }
 }
